require 'fileutils'

BeforeAll do
  $api_key = "12312312312312312312312312312312"
  Maze.config.receive_no_requests_wait = 15

  # Setup a 3 minute timeout for receiving requests is STRESS_TEST env var is set
  Maze.config.receive_requests_wait = 180 unless ENV['STRESS_TEST'].nil?

  # Additional require MacOS configuration
  if Maze.config.os == 'macos'
    # The default macOS Crash Reporter "#{app_name} quit unexpectedly" alert grabs focus which can cause tests to flake.
    # This option, which appears to have been introduced in macOS 10.11, displays a notification instead of the alert.
    `defaults write com.apple.CrashReporter UseUNC 1`

    fixture_dir = 'features/fixtures/macos/output'
    app_dir = '/Applications'
    zip_name = "#{Maze.config.app}.zip"
    app_name = "#{Maze.config.app}.app"

    # If built app file already exists, skip unzip
    unless File.exist?("#{fixture_dir}/#{app_name}")
      raise Exception, 'Test fixture build archive not found' unless File.file?("#{fixture_dir}/#{zip_name}")
      `cd #{fixture_dir} && unzip #{zip_name}`
    end

    FileUtils.mv("#{fixture_dir}/#{app_name}", "#{app_dir}/#{app_name}")
  end
end

AfterAll do
  # Additional require MacOS configuration
  if Maze.config.os == 'macos'
    app_dir = '/Applications'
    app_name = "#{Maze.config.app}.app"
    FileUtils.rm_rf("#{app_dir}/#{app_name}")
  end
end

def skip_below(os, version)
  skip_this_scenario("Skipping scenario") if Maze.driver.capabilities['platformName'] == os and Maze.config.os_version < version
end

Before('@skip_below_ios_11') do |scenario|
  skip_below('iOS', 11)
end

Before('@skip_below_ios_13') do |scenario|
  skip_below('iOS', 13)
end

Before('@skip_macos') do |scenario|
  skip_this_scenario("Skipping scenario") if Maze.driver.capabilities['platformName'] == 'Mac'
end

# Skip stress tests unless STRESS_TEST env var is set
Before('@stress_test') do |_scenario|
  skip_this_scenario('Skipping: Run is not configured for stress tests') if ENV['STRESS_TEST'].nil?
end
